//
//  MYScanPlugin.swift
//  moyoung_ble_plugin
//
//  Created by 魔样科技 on 2022/5/21.
//

import Foundation
import Flutter
import CRPSmartBand

enum HardWareType: Int {
    case None = 0;
    case Nordic = 1;
    case HTX = 2;
    case RealTek = 3;
    case Goodix = 4;
}

public class MYScanPlugin: NSObject {
    var myDiscovery:CRPDiscovery!
    var deviceState: CRPState = .connecting
    lazy var discoverys: [CRPDiscovery] = [CRPDiscovery]()
    lazy var notifications: [NotificationType] = [.phone, .messages, .wechat, .qq, .facebook, .twitter, .instagram, .skype, .whatsApp, .line, .kakaoTalk, .gmail, .messenger, .zalo, .telegram, .viber, .others]
    
    var deviceCurrentVersion: String = ""
    var deviceNewVersionInfo: CRPSmartBand.newVersionInfo = newVersionInfo(type: .normal, version: "", log: "", logEN: "", mcu: 0, md5: "", fileUrl: "")
    var hardWareType: HardWareType = .None
    
    
    /// flutter:   接收 扫描到的设备的监听
    var scannedDevicesEventSink: FlutterEventSink?
    /// flutter:    接收设备连接状态的监听
    var connectionStateEventSink: FlutterEventSink?
    /// flutter:    接收实时的计步数据的监听
    var stepEventSink: FlutterEventSink?
    /// flutter:    接收运动的监听
    var trainingStateEventSink: FlutterEventSink?
    /// flutter:   体温的监听
    var temperatureEventSink: FlutterEventSink?
    /// flutter:    接收设备电量的监听
    var batteryEventSink: FlutterEventSink?
    /// flutter:    天气的监听
    var weatherEventSink: FlutterEventSink?
    /// flutter:    过去两天的步数分类统计的监听
    var stepsDetailEventSink: FlutterEventSink?
    /// flutter:    睡眠的监听
    /// static const int sleepChange = 1;static const int historySleepChange = 2;
    var sleepEventSink: FlutterEventSink?
    /// flutter:    固件升级的监听
    var firmwareUpgradeEventSink: FlutterEventSink?
    /// flutter:    接收⼼率测量结果(单次测量⼼率)的监听
    /// static const int measuring = 1;static const int onceMeasureComplete = 2;static const int heartRate = 3;static const int measureComplete = 4;static const int hourMeasureResult = 5;static const int measureResult = 6;
    var heartRateEventSink: FlutterEventSink?
    /// flutter:    接收⾎压测量结果
    var bloodPressureEventSink: FlutterEventSink?
    /// flutter:    接收⾎氧测量结果的监听
    var bloodOxygenEventSink: FlutterEventSink?
    /// 相机的监听
    var cameraEventSink: FlutterEventSink?
    /// flutter:    手机的监听
    var phoneEventSink: FlutterEventSink?
    /// flutter:    设备RSSI的监听
    var deviceRSSIEventSink: FlutterEventSink?
    /// flutter:
    var fileTransEventSink: FlutterEventSink?
    /// flutter:
    var wfFileTransEventSink: FlutterEventSink?
    /// flutter:    心电图的监听
    var ecgEventSink: FlutterEventSink?
    /// flutter:    联系人头像的监听
    var contactAvatarEventSink: FlutterEventSink?
    /// flutter:    联系人的监听
    var contactEventSink: FlutterEventSink?
    /// flutter:    省电模式的监听
    var batterySavingEventSink: FlutterEventSink?
    /// flutter:    锻炼的监听
    var trainEventSink: FlutterEventSink?
    
    /// 吃药提醒数据闭包
    var receiveMedicineInfoClosure:((_ max: Int,_ model: CRPMedicineReminderModel?) -> Void)!
    
    /// 收到吃药提醒数据
    /// - Parameter closure: _ max: Int,_ model: CRPMedicineReminderModel
    func returnReceiveMedicineInfoClosure(closure : @escaping ((_ max: Int,_ model: CRPMedicineReminderModel?) -> Void)) {
        self.receiveMedicineInfoClosure = closure
    }
    
    class func setupEventChannel(name: String, registrar: FlutterPluginRegistrar, instance: (FlutterStreamHandler & NSObjectProtocol)?) {
        let channel = FlutterEventChannel(name: name, binaryMessenger: registrar.messenger())
        channel.setStreamHandler(instance)
    }
    
}

extension MYScanPlugin: FlutterPlugin, FlutterStreamHandler, CRPManagerDelegate {
    
    //MARK: - eventSink delegate
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        scannedDevicesEventSink = events
        connectionStateEventSink = events
        stepEventSink = events
        trainingStateEventSink = events
        temperatureEventSink = events
        batteryEventSink = events
        weatherEventSink = events
        stepsDetailEventSink = events
        sleepEventSink = events
        firmwareUpgradeEventSink = events
        heartRateEventSink = events
        bloodPressureEventSink = events
        bloodOxygenEventSink = events
        cameraEventSink = events
        phoneEventSink = events
        deviceRSSIEventSink = events
        fileTransEventSink = events
        wfFileTransEventSink = events
        ecgEventSink = events
        contactAvatarEventSink = events
        contactEventSink = events
        batterySavingEventSink = events
        trainEventSink = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        scannedDevicesEventSink = nil
        connectionStateEventSink = nil
        stepEventSink = nil
        trainingStateEventSink = nil
        temperatureEventSink = nil
        batteryEventSink = nil
        weatherEventSink = nil
        stepsDetailEventSink = nil
        sleepEventSink = nil
        firmwareUpgradeEventSink = nil
        heartRateEventSink = nil
        bloodPressureEventSink = nil
        bloodOxygenEventSink = nil
        cameraEventSink = nil
        phoneEventSink = nil
        deviceRSSIEventSink = nil
        fileTransEventSink = nil
        wfFileTransEventSink = nil
        ecgEventSink = nil
        contactAvatarEventSink = nil
        contactEventSink = nil
        batterySavingEventSink = nil
        trainEventSink = nil
        return nil
    }
    
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        //方法的通道
        let scanChannel = FlutterMethodChannel(name: METHOD_SCAN_DEVICE, binaryMessenger: registrar.messenger())
        let connectChannel = FlutterMethodChannel(name: METHOD_CONNECTION, binaryMessenger: registrar.messenger())
        //创建类对象
        let instance = MYScanPlugin()
        CRPSmartBandSDK.sharedInstance.delegate = instance
        //添加注册
        registrar.addMethodCallDelegate(instance, channel: scanChannel)
        registrar.addMethodCallDelegate(instance, channel: connectChannel)
        let eventChannelNames = [EVENT_SCAN_DEVICE, EVENT_CONNECTION_STATE, EVENT_STEP_CHANGE, EVENT_TRAINING_STATE, EVENT_TEMP_CHANGE, EVENT_DEVICE_BATTERY, EVENT_WEATHER_CHANGE, EVENT_STEPS_DETAIL, EVENT_SLEEP_CHANGE, EVENT_FIRMWARE_UPGRADE, EVENT_HEART_RATE, EVENT_BLOOD_PRESSURE, EVENT_BLOOD_OXYGEN, EVENT_BLOOD_CAMERA, EVENT_PHONE, EVENT_DEVICE_RSSI, EVENT_FILE_TRANS, EVENT_WF_FILE_TRANS, EVENT_EGC, EVENT_CONTACT_AVATAR, EVENT_CONTACT, EVENT_BATTERY_SAVING, EVENT_TRAIN]
        eventChannelNames.forEach { name in
            setupEventChannel(name: name, registrar: registrar, instance: instance)
        }
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        var receivedDic: [String: Any] = [:]
        if call.arguments is [String: Any] {
            receivedDic = call.arguments as! [String : Any]
        }
 
//        print("flutter方法是：\(call.method)，flutter发送过来的数据是：\(receivedDic)")
        
        //MARK: - 扫描连接 -

        //开始扫描
        if call.method == "startScan" {
            CRPSmartBandSDK.sharedInstance.scan(TimeInterval(receivedDic["scanPeriod"] as! Int)) {[weak self] datas in
                guard let self = self else { return  }
                for p in datas {
                    if p.localName?.isEmpty == true {
                        return
                    }
                    self.discoverys.append(p)
                    var param: [String: Any] = [:]
                    param["isCompleted"] = false
                    param["address"] = p.mac
                    param["mRssi"] = p.RSSI
                    param["mScanRecord"] = [Int]()
                    param["name"] = p.localName
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    result(true)
                    self.scannedDevicesEventSink?(str)
                }
                
            } completionHandler: {[weak self] (datas, error) in
                guard let self = self else { return  }
                var param: [String: Any] = [:]
                param["isCompleted"] = true
                param["address"] = ""
                param["mRssi"] = ""
                param["mScanRecord"] = [Int]()
                param["name"] = ""
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(true)
                self.scannedDevicesEventSink?(str)
                switch error {
                    
                case .none: break
                    //TODO: -
                case .disconnected: break
                    //TODO: -此处为断开连接后，是否进行重连
                case .busy: break
                    //TODO: -此处为设备已经被连接，该进行怎样的逻辑
                case .timeout: break
                    //TODO: -此处为扫描超时咯
                case .interrupted: break
                    //TODO: -此处为扫描被中断
                case .internalError: break
                    //TODO: -此处为SDK内部错误
                case .noCentralManagerSet: break
                    //TODO: -此处为没有蓝牙设备中心
                case .other: break
                    //TODO: -此处为其他错误
                default: break
                    //TODO: -此处为默认
                }
            }

        }
        else if call.method == "cancelScan" {
            //取消扫描
            CRPSmartBandSDK.sharedInstance.interruptScan()
        }
        else if call.method == "isConnected" {
            var isConnected = false
            if self.deviceState == .connected {
                isConnected = true
            }
            result(isConnected)
        }
        else if call.method == "connect" {
            //连接
            for p in self.discoverys {
                if p.mac == receivedDic["address"] as? String {
                    myDiscovery = p
                    CRPSmartBandSDK.sharedInstance.connet(p)
                    return
                }
            }
        }
        else if call.method == "disconnect" {
            //断开连接
            CRPSmartBandSDK.sharedInstance.disConnet()
            result(true)
        }
        //MARK: - 同步时间 -
        else if call.method == "queryTime" {
            //同步时间
            CRPSmartBandSDK.sharedInstance.setTime()
        }
        //MARK: - 设置时间制式 -
        else if call.method == "sendTimeSystem" {
            //设置时间制
            CRPSmartBandSDK.sharedInstance.setTimeFormat(receivedDic["timeSystemType"] as! Int)
        }
        else if call.method == "queryTimeSystem" {
            //查询时间制式
            CRPSmartBandSDK.sharedInstance.getTimeformat { timeFormat, error in
                result(timeFormat)
            }
        }
        //MARK: - 查询手环电量 -
        else if call.method == "queryDeviceBattery" {
            //查询手环电量
            CRPSmartBandSDK.sharedInstance.getBattery { battery, error in
                //TODO: flutter端是监听接收返回的
                var param: [String: Any] = [:]
                param["type"] = 2
                param["subscribe"] = false
                param["deviceBattery"] = battery
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.batteryEventSink?(str)
            }
        }
        else if call.method == "subscribeDeviceBattery" {
            //订阅手环电量
            //TODO: iOS端暂无
        }
        //MARK: - 固件升级 -
        else if call.method == "startOTA" {
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["otaBean"] as! String)
            switch self.hardWareType {
            case .HTX:
                CRPSmartBandSDK.sharedInstance.startHTXOTA(info: self.deviceNewVersionInfo, upgradeMac: dic["address"] as! String, isUser: true)
            case .RealTek:
                CRPSmartBandSDK.sharedInstance.startRealTekUpgrade(info: self.deviceNewVersionInfo)
            case .Goodix:
                CRPSmartBandSDK.sharedInstance.startGoodixUpgradeFromFile(zipPath: self.deviceNewVersionInfo.fileUrl ?? "")
            case .Nordic:
                CRPSmartBandSDK.sharedInstance.startUpgrede(info: self.deviceNewVersionInfo)
            default: break
            }
        }
        else if call.method == "abortOTA" {
            switch self.hardWareType {
            case .HTX:
                CRPSmartBandSDK.sharedInstance.stopHTXOTA()
            case .RealTek:
                break
            case .Goodix:
                break
            case .Nordic:
                CRPSmartBandSDK.sharedInstance.stopUpgrade()
            default:
                break
            }
        }
        else if call.method == "queryDeviceOtaStatus" {
            CRPSmartBandSDK.sharedInstance.checkDFUState { state, error in
                //(DFU不为0时既⼿环处于升级状态
                var status: Int = 0
                if state != "0" {
                    status = 1
                }
                result(status)
            }
        }

        else if call.method == "queryFirmwareVersion" {
            //查询⼿环当前固件版本
            CRPSmartBandSDK.sharedInstance.getSoftver {(softver, error) in
                self.deviceCurrentVersion = softver
                result(softver)
            }
        }
        else if call.method == "queryHsOtaAddress" {
            //
            CRPSmartBandSDK.sharedInstance.getOTAMac { address, error in
                result(address)
            }
        }
        //MARK: - iOS端暂无
        else if call.method == "enableHsOta" {
            //

        }
        //MARK: - iOS端暂无
        else if call.method == "queryOtaType" {
            //

        }
        else if call.method == "checkFirmwareVersion" {
            //查询新固件
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["firmwareVersion"] as! String)
            self.deviceCurrentVersion = dic["version"] as! String
            CRPSmartBandSDK.sharedInstance.checkLatest(self.deviceCurrentVersion, self.myDiscovery.mac ?? "") { newVerInfo, newVerTpInfo, error in
                var param: [String: Any] = [:]
                if newVerInfo == nil {
                    param["isLatestVersion"] = true
                }
                else {
                    self.deviceNewVersionInfo = newVerInfo ?? newVersionInfo()
                    if newVerInfo != nil {
                        var firmwareVersionInfo: [String: Any] = [:]
                        firmwareVersionInfo["changeNotes"] = newVerInfo?.log
                        firmwareVersionInfo["changeNotesEn"] = newVerInfo?.logEN
                        firmwareVersionInfo["mcu"] = newVerInfo?.mcu
                        firmwareVersionInfo["tpUpgrade"] = false
                        firmwareVersionInfo["type"] = newVerInfo?.type?.rawValue
                        firmwareVersionInfo["version"] = newVerInfo?.version
                        param["firmwareVersionInfo"] = firmwareVersionInfo
                        
                        if newVerInfo?.mcu == 4 || newVerInfo?.mcu == 8 {
                            self.hardWareType = .HTX
                        }
                        else if newVerInfo?.mcu == 7 {
                            self.hardWareType = .RealTek
                        }
                        else if newVerInfo?.mcu == 10 {
                            self.hardWareType = .Goodix
                        }
                        else {
                            self.hardWareType = .Nordic
                        }
                    }
                    param["isLatestVersion"] = self.deviceCurrentVersion == newVerInfo?.version ? true : false
                }
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
                
            }
        }
        //MARK: - 用户信息 -
        else if call.method == "sendUserInfo" {
            //设置⽤户信息
            let userInfoDic: Dictionary = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["userInfo"] as! String)
            let model: ProfileModel = ProfileModel(height: userInfoDic["height"] as! Int, weight: userInfoDic["weight"] as! Int, age: userInfoDic["age"] as! Int, gender: GenderOption(rawValue: GenderOption.RawValue(userInfoDic["gender"] as! Int)) ?? .male)
            CRPSmartBandSDK.sharedInstance.setProfile(model)
        }
        else if call.method == "sendStepLength" {
            //设置步⻓
            CRPSmartBandSDK.sharedInstance.setStepLength(length: receivedDic["stepLength"] as! Int)
        }
        //MARK: - 天气 -
        else if call.method == "sendTodayWeather" {
            //设置今⽇天⽓
            let weatherDic: Dictionary = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["todayWeatherInfo"] as! String)
            let todayWeather:CRPSmartBand.weather = weather.init(type: weatherDic["weatherId"] as! Int, temp: weatherDic["temp"] as! Int , pm25: weatherDic["pm25"] as! Int, festival: weatherDic["festival"] as! String, city: weatherDic["city"] as! String)
            CRPSmartBandSDK.sharedInstance.setWeather(todayWeather)
        }
        else if call.method == "sendFutureWeather" {
            //设置未来7⽇天⽓
            let weatherDic: Dictionary = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["futureWeatherInfo"] as! String)
            let weathers: [[String: Any]] = weatherDic["future"] as! [[String : Any]]
            var models: [forecastWeather] = []
            for weather in weathers {
                let futureWeather:CRPSmartBand.forecastWeather = forecastWeather.init(type: weather["weatherId"] as! Int, maxTemp: weather["highTemperature"] as! Int, minTemp: weather["lowTemperature"] as! Int)
                models.append(futureWeather)
            }
            CRPSmartBandSDK.sharedInstance.setForecastWeather(models)
        }
        else if call.method == "sendLocalCity" {
            //设置本地城市
            CRPSmartBandSDK.sharedInstance.sendCityName(receivedDic["city"] as! String)
        }
        //MARK: - 体温 -
        else if call.method == "queryUnitSystem" {
            //获取体温单位
            CRPSmartBandSDK.sharedInstance.getTemperatureUnit { temperature, error in
                result(temperature)

            }
        }
        else if call.method == "sendUnitSystem" {
            //设置体温单位
            CRPSmartBandSDK.sharedInstance.setTemperatureUnit(receivedDic["metricSystemType"] as! Int)
        }
        //MARK: - 翻腕亮屏 -
        else if call.method == "sendQuickView" {
            //设置翻腕亮屏状态
            CRPSmartBandSDK.sharedInstance.setQuickView(receivedDic["quickViewState"] as! Bool)
        }
        else if call.method == "queryQuickView" {
            //查询⼿环翻腕亮屏开启状态,0: 关闭 1： 开启
            CRPSmartBandSDK.sharedInstance.getQuickView { state, error in
                result(Bool(truncating: state as NSNumber))
            }
        }
        else if call.method == "sendQuickViewTime" {
            //设置翻腕亮屏时间
            let periodTimeDic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["periodTimeInfo"] as! String)
            let model = periodTimeModel.init(startHour: periodTimeDic["startHour"] as! Int, startMin: periodTimeDic["startMinute"] as! Int, endHour: periodTimeDic["endHour"] as! Int, endMin: periodTimeDic["endMinute"] as! Int)
            CRPSmartBandSDK.sharedInstance.setQuickViewTime(model)
        }
        else if call.method == "queryQuickViewTime" {
            //获取翻腕亮屏时间
            CRPSmartBandSDK.sharedInstance.getQuickViewTime { model, error in
                var param: [String: Any] = [:]
                
                var periodTimeInfo: [String: Any] = [:]
                periodTimeInfo["startHour"] = model.startHour
                periodTimeInfo["startMinute"] = model.startMin
                periodTimeInfo["endHour"] = model.endHour
                periodTimeInfo["endMinute"] = model.endMin
                // int DO_NOT_DISTRUB_TYPE = 1;int QUICK_VIEW_TYPE = 2;
                param["periodTimeType"] = 2
                param["periodTimeInfo"] = periodTimeInfo
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
                
            }
        }
        //MARK: - 目标步数 -
        else if call.method == "querySteps" {
            //今天的步数
            CRPSmartBandSDK.sharedInstance.getSteps { model, error in
               //TODO: flutter是通过监听接收的
                var dic: [String: Any] = [:]
                dic["calories"] = model.calory
                dic["distance"] = model.distance
                dic["steps"] = model.steps
                dic["time"] = model.time
                
                var param: [String: Any] = [:]
                param["timeType"] = 0
                param["stepsInfo"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.stepEventSink?(str)
            }
        }
        else if call.method == "queryHistorySteps" {
            let type: Int = receivedDic["historyTimeType"] as! Int
            //同步历史步数
            CRPSmartBandSDK.sharedInstance.getAllData { stepModels, sleepModels, error in
                //⼿环可保存前两天的计步数据和睡眠数据(allDataHandler中包含stepModels和sleepModels,数组的第⼀个为昨天数据，第⼆个为前天数据)
                //TODO: flutter是通过监听接收的
                if type == 0 {
                    //昨天的步数

                    let stepModel0 = stepModels[0]
                    var stepDic0: [String: Any] = [:]
                    stepDic0["calories"] = stepModel0.calory
                    stepDic0["distance"] = stepModel0.distance
                    stepDic0["steps"] = stepModel0.steps
                    stepDic0["time"] = stepModel0.time
                    
                    var stepParam: [String: Any] = [:]
                    stepParam["timeType"] = 1
                    stepParam["stepsInfo"] = stepDic0
                    let stepStr = SystemAuth.getJsonStringFromBasicTypeData(data: stepParam)
                    self.stepEventSink?(stepStr)
                }
                else if type == 1 {
                    //前天的步数
                    let stepModel0 = stepModels[1]
                    var stepDic0: [String: Any] = [:]
                    stepDic0["calories"] = stepModel0.calory
                    stepDic0["distance"] = stepModel0.distance
                    stepDic0["steps"] = stepModel0.steps
                    stepDic0["time"] = stepModel0.time
                    
                    var stepParam: [String: Any] = [:]
                    stepParam["timeType"] = 2
                    stepParam["stepsInfo"] = stepDic0
                    let stepStr = SystemAuth.getJsonStringFromBasicTypeData(data: stepParam)
                    self.stepEventSink?(stepStr)
                }
            }
        }
        else if call.method == "queryStepsDetail" {
            //查询步数详情
            //TODO: - flutter是监听接收
            let dateType = receivedDic["stepsDetailDateType"] as! Int
            if dateType == 0 {
                CRPSmartBandSDK.sharedInstance.get24HourSteps { datas, error in
                    var param: [String: Any] = [:]
                    param["dateType"] = 0
                    param["stepsList"] = datas
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.stepsDetailEventSink?(str)
                }
            }
            else {
                CRPSmartBandSDK.sharedInstance.getAgo24HourSteps { datas, error in
                    var param: [String: Any] = [:]
                    param["dateType"] = 1
                    param["stepsList"] = datas
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.stepsDetailEventSink?(str)
                }
            }
        }
        else if call.method == "sendGoalSteps" {
            //设置目标步数
            CRPSmartBandSDK.sharedInstance.setGoal(receivedDic["goalSteps"] as! Int)
        }
        else if call.method == "queryGoalSteps" {
            //查询⽬标步数
            CRPSmartBandSDK.sharedInstance.getGoal { goal, error in
                result(goal)
            }
        }
        //MARK: - 表盘 -
        else if call.method == "sendDisplayWatchFace" {
            //切换⼿环表盘
            CRPSmartBandSDK.sharedInstance.setDial(receivedDic["watchFaceType"] as! Int)
        }
        else if call.method == "queryDisplayWatchFace" {
            //查询⼿环正在使⽤的表盘序号
            CRPSmartBandSDK.sharedInstance.getDial { number, error in
                result(number)
            }
        }
        else if call.method == "queryWatchFaceLayout" {
            //获取指定表盘布局
            CRPSmartBandSDK.sharedInstance.getScreenContent { content, imageSize, i, error in
                var param: [String: Any] = [:]
                param["backgroundPictureMd5"] = content.md5
                param["compressionType"] = ""
                param["height"] = imageSize.originalHeight
                //hex转int
//                let a = Scanner.init(string: content.contentColor.hexString!)
//                var b: UInt32 = 0
//                if withUnsafeMutablePointer(to: &b, { a.scanHexInt32($0)
//                }){
//                    print(Int(b))
//                }
//                let colorString: String = content.contentColor.hexString ?? ""
//                let colorNumber: Int = colorString.hexToDecamail()
                param["textColor"] = content.contentColor.hexString?.hexToDecamail()
                
                param["thumHeight"] = imageSize.abbreviatedHeight
                param["thumWidth"] = imageSize.abbreviatedWidth
                param["timeBottomContent"] = content.underContent.rawValue
                param["timePosition"] = content.position.rawValue
                param["timeTopContent"] = content.upperContent.rawValue
                param["width"] = imageSize.originalWidth
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
                
            }
        }
        else if call.method == "sendWatchFaceLayout" {
            //修改⾃定义表盘布局
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["watchFaceLayoutInfo"] as! String)

            let hexColor = String(dic["textColor"] as! Int, radix: 16, uppercase: true)
            let textColor = UIColor.init(hexString: hexColor)
            
            let content = ScreenContent(position: ContentPosition(rawValue: dic["timePosition"] as! Int)!, upperContent: Content(rawValue: dic["timeTopContent"] as! Int)!, underContent: Content(rawValue: dic["timeBottomContent"] as! Int)!, contentColor: textColor, md5: dic["backgroundPictureMd5"] as! String)
            
            CRPSmartBandSDK.sharedInstance.setupScreenContent(content: content)
            
        }
        else if call.method == "querySupportWatchFace" {
            //获取⽀持摆盘类型
            CRPSmartBandSDK.sharedInstance.getWatchFaceSupportModel { model, error in
                var param: [String: Any] = [:]
                param["displayWatchFace"] = model.currentID
                param["supportWatchFaceList"] = model.supportModel
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        else if call.method == "queryWatchFaceStore" {
            //获取表盘市场,根据表盘的⽀持类型获取能可以更换的表盘列表(分⻚获取)
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["watchFaceStoreBean"] as! String)
            CRPSmartBandSDK.sharedInstance.getWatchFaceInfo(dic["watchFaceSupportList"] as! [Int], currentPage: dic["pageIndex"] as! Int, perPage: dic["pageCount"] as! Int) { datas, currentPage, perPage, error in
                var param: [String: Any] = [:]
                
                var watchFaceStore: [String: Any] = [:]
                watchFaceStore["prePage"] = perPage
                watchFaceStore["pageIndex"] = currentPage
                var list: [[String: Any]] = [[String: Any]]()
                for data in datas {
                    var info: [String: Any] = [:]
                    info["id"] = data.id
                    info["preview"] = data.imageUrl
                    info["file"] = data.fileUrl
                    list.append(info)
                }
                watchFaceStore["list"] = list
                watchFaceStore["total"] = list.count
                
                param["watchFaceStore"] = watchFaceStore
                param["error"] = String(describing: error)
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        else if call.method == "queryWatchFaceOfID" {
            //获取表盘ID的表盘信息
            let currentId = receivedDic["id"] as! Int
            CRPSmartBandSDK.sharedInstance.getWatchFaceInfoByID(currentId) { datas, currentPage, perPage, error in
                var param: [String: Any] = [:]
                var info: [String: Any] = [:]
                for data in datas {
                    if data.id == currentId {
                        info["id"] = data.id
                        info["preview"] = data.imageUrl
                        info["file"] = data.fileUrl
                    }
                }
                if info.keys.count == 0 {
                    info["id"] = currentId
                    info["preview"] = ""
                    info["file"] = ""
                }
                
                param["watchFace"] = info
                param["error"] = String(describing: error)
                param["code"] = 0
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        else if call.method == "sendWatchFaceBackground" {
            //设置表盘背景
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["watchFaceBackgroundInfo"] as! String)
            let imageSize = ScreenImageSize(originalWidth: dic["width"] as! Int, originalHeight: dic["height"] as! Int, abbreviatedWidth: dic["thumbWidth"] as! Int, abbreviatedHeight: dic["thumbHeight"] as! Int)
            // Uint8List bitmap;Uint8List thumbBitmap;
            let image = UIImage(data: Data(dic["bitmap"] as! Array))!
            CRPSmartBandSDK.sharedInstance.startChangeScreen(image, imageSize, false, 0)
            
        }
        else if call.method == "sendWatchFace" {
            //将新表盘的表盘文件发送给手表，在此过程中手表会重新启动。这个转移进程由CRPWatchFaceTransListener。
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["sendWatchFaceBean"] as! String)
            let info: [String: Any] = dic["watchFaceFlutterBean"] as! [String : Any]
            CRPSmartBandSDK.sharedInstance.startChangeWathcFaceFromFile(path: info["file"] as! String)
        }
        
        
        
        
        //MARK: - 闹钟 -
        else if call.method == "sendAlarm" {
            //设置手环闹钟
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["alarmClockInfo"] as! String)
            
            let repeatMode: Int = dic["repeatMode"] as! Int
            var type: CRPSmartBand.AlarmType = .once
            var weekday: [Int] = []
            if repeatMode == 0 {
                type = .once
                weekday = []
            }
            else if repeatMode == 1 {
                type = .weekly
                weekday = [0]
            }
            else if repeatMode == 2 {
                type = .weekly
                weekday = [1]
            }
            else if repeatMode == 4 {
                type = .weekly
                weekday = [3]
            }
            else if repeatMode == 8 {
                type = .weekly
                weekday = [4]
            }
            else if repeatMode == 16 {
                type = .weekly
                weekday = [5]
            }
            else if repeatMode == 32 {
                type = .weekly
                weekday = [6]
            }
            else if repeatMode == 64 {
                type = .weekly
                weekday = [7]
            }
            else {
                type = .everyday
            }
            
            let model = AlarmModel(id: dic["id"] as! Int, enable: dic["enable"] as! Int, type: type, hour: dic["hour"] as! Int, minute: dic["minute"] as! Int, year:0, month: 0, day: 0, weekday: weekday)
            CRPSmartBandSDK.sharedInstance.setAlarm(model)
        }
        else if call.method == "queryAllAlarm" {
            //查询所有闹钟
            CRPSmartBandSDK.sharedInstance.getAlarms { alarms, error in
                var arr: [[String: Any]] = []
                for alarm in alarms {
                    var param: [String: Any] = [:]
                    param["id"] = alarm.id
                    param["enable"] = Bool(truncating: alarm.enable as NSNumber)
                    param["hour"] = alarm.hour
                    param["minute"] = alarm.minute
                    
                    var repeatMode: Int = 0
                    switch alarm.type {
                    case .once:
                        repeatMode = 0
                    case .everyday:
                        repeatMode = 127
                    case .weekly:
                        repeatMode = alarm.weekday[0]
                    default:
                        repeatMode = 0
                    }
                    param["repeatMode"] = repeatMode
                    
    //                param["year"] = alarm.year
    //                param["month"] = alarm.month
    //                param["day"] = alarm.day
    //                param["weekday"] = alarm.weekday
                    arr.append(param)
                }
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: arr)
                result(str)
            }
        }
        //MARK: - 心率 -
        else if call.method == "queryLastDynamicRate" {
            //查询上次动态⼼率测量结果,receiveHeartRateAll回调
            //TODO: - 这里要去代理里面加监听，然后组合一下数据传给flutter
            CRPSmartBandSDK.sharedInstance.getHeartData()
        }
        else if call.method == "enableTimingMeasureHeartRate" {
            //开始定时测量心率，正数为开启
            CRPSmartBandSDK.sharedInstance.set24HourHeartRate(receivedDic["interval"] as! Int)
        }
        else if call.method == "disableTimingMeasureHeartRate" {
            //关闭定时测量心率，0为关闭
            CRPSmartBandSDK.sharedInstance.set24HourHeartRate(0)
        }
        else if call.method == "queryTimingMeasureHeartRate" {
            //获取24⼩时定时测量间隔时间
            CRPSmartBandSDK.sharedInstance.get24HourHeartRateInterval { headtRateInterval, error in
                result(headtRateInterval)
            }
        }
        else if call.method == "queryTodayHeartRate" {
            //查询今天定时测量⼼率数据
            //TODO: - flutter监听接收
            /*
             TodayHeartRateType：
             | timingMeasureHeartRate       | **allDayHeartRate**            |
             | ---------------------------- | ------------------------------ |
             | Timed heart rate measurement | 24-hour continuous measurement |
             */
            CRPSmartBandSDK.sharedInstance.get24HourHeartRate { datas, error in
                
                var dic: [String: Any] = [:]
                dic["heartRateList"] = datas
                
                var param: [String: Any] = [:]
                param["type"] = 5
                param["hour24MeasureResult"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.heartRateEventSink?(str)
            }
        }
        else if call.method == "queryPastHeartRate" {
            //查询过去的定时测量⼼率数据
            CRPSmartBandSDK.sharedInstance.getAgo24HourHeartRate { datas, error in
                //TODO: - 这里会返回[int],flutter用监听接收
                var dic: [String: Any] = [:]
                dic["heartRateList"] = datas
                
                var param: [String: Any] = [:]
                param["type"] = 5
                param["hour24MeasureResult"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.heartRateEventSink?(str)
            }
        }
        else if call.method == "queryTrainingHeartRate" {
            //获取运动数据,测量量结果包括⼼⼼率和卡路⾥等其它运动相关数据,保存最近三次
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.getSportData({ models, error in
                
                var list: [[String: Any]] = [[String: Any]]()
                for model in models {
                    var dic: [String: Any] = [:]
                    dic["type"] = model.type.rawValue
                    dic["startTime"] = model.startTime
                    dic["endTime"] = model.endTime
                    dic["validTime"] = model.vaildTime
                    dic["steps"] = model.step
                    dic["distance"] = model.distance
                    dic["calories"] = model.kcal
                    list.append(dic)
                }

                var param: [String: Any] = [:]
                param["type"] = 6
                param["trainingList"] = list
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.heartRateEventSink?(str)
            })
        }
        else if call.method == "startMeasureOnceHeartRate" {
            //开启静态⼼率单次测量,测量结果通过receiveHeartRate(_ heartRate: Int)回调
            CRPSmartBandSDK.sharedInstance.setStartSingleHR()
        }
        else if call.method == "stopMeasureOnceHeartRate" {
            //结束静态⼼率单次测量,测量结果通过receiveHeartRate(_ heartRate: Int)回调
            //TODO: - 这里方法走完会进入  receiveHeartRate代理，flutter监听接收
            CRPSmartBandSDK.sharedInstance.setStopSingleHR()
        }
        else if call.method == "queryHistoryHeartRate" {
            //获取单次⼼率测量历史记录
            CRPSmartBandSDK.sharedInstance.getHeartRecordData { heartRecordDatas, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                var list: [[String: Any]] = [[String: Any]]()
                for data in heartRecordDatas {
                    var dic: [String: Any] = [:]
                    dic["hr"] = data.value
                    dic["date"] = String(data.time)
                    list.append(dic)
                }
                param["type"] = 3
                param["historyHrList"] = list
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.heartRateEventSink?(str)
            }
        }
        //MARK: - 血压 -
        else if call.method == "startMeasureBloodPressure" {
            //开始测量⾎压，点了以后，就会开始走代理receiveBloodPressure，
            //TODO: - 这里会调用回调receiveBloodPressure，flutter监听接收
            CRPSmartBandSDK.sharedInstance.setStartBlood()
        }
        else if call.method == "stopMeasureBloodPressure" {
            //停⽌测量⾎压,测量结果通过receiveBloodPressure回调
            CRPSmartBandSDK.sharedInstance.setStopBlood()
        }
        else if call.method == "enableContinueBloodPressure" {
            //设置全天⾎压
            CRPSmartBandSDK.sharedInstance.setFullDayBPStatus(open: true)
        }
        else if call.method == "disableContinueBloodPressure" {
            //设置全天⾎压
            CRPSmartBandSDK.sharedInstance.setFullDayBPStatus(open: false)
        }
        else if call.method == "queryContinueBloodPressureState" {
            //获取全天⾎压状态
            CRPSmartBandSDK.sharedInstance.getFullDayBPStatus { status, error in
               //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 1
                param["continueState"] = Bool(truncating: status as NSNumber)
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodPressureEventSink?(str)
            }
        }
        //查询最近24小时血压
        //Android通过CRPBloodPressureChangeListener回调。onContinueBloodPressure (CRPBloodPressureInfo信息);
        else if call.method == "queryLast24HourBloodPressure" {
            //获取全天⾎压数据
            CRPSmartBandSDK.sharedInstance.getFullDayBPData { datas, error in
                //TODO: - flutter监听接收, 数据对不上
                var dic: [String: Any] = [:]
                dic["startTime"] = 0
                dic["timeInterval"] = 0
                
                var param: [String: Any] = [:]
                param["type"] = 4
                param["continueBp"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodPressureEventSink?(str)
            }
        }
        else if call.method == "queryHistoryBloodPressure" {
            //获取单次⾎压测量历史记录
            //Android通过CRPBloodPressureChangeListener回调。onHistoryBloodPressure列表(列表);
            CRPSmartBandSDK.sharedInstance.getBPRecordData { datas, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                var list: [[String: Any]] = [[String: Any]]()
                for data in datas {
                    var info: [String: Any] = [:]
                    info["SBP"] = data.SBP
                    info["DBP"] = data.DBP
                    info["date"] = String(data.time)
                    list.append(info)
                }
                param["type"] = 3
                param["historyBpList"] = list
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodPressureEventSink?(str)
            }
        }
        //MARK: - 血氧 -
        else if call.method == "startMeasureBloodOxygen" {
            //开始测量⾎氧
            //TODO: - 好像时间到了以后也会走代理receiveSpO2，flutter监听接收
            CRPSmartBandSDK.sharedInstance.setStartSpO2()
        }
        else if call.method == "stopMeasureBloodOxygen" {
            //停⽌测量⾎氧
            //TODO: - 返回值在receiveSpO2代理，flutter监听接收
            CRPSmartBandSDK.sharedInstance.setStopSpO2()
        }
        else if call.method == "enableTimingMeasureBloodOxygen" {
            //开启定时测量血氧
            CRPSmartBandSDK.sharedInstance.setAutoO2(receivedDic["interval"] as! Int)
        }
        else if call.method == "disableTimingMeasureBloodOxygen" {
            //禁用计时测量血氧
            CRPSmartBandSDK.sharedInstance.setAutoO2(0)
        }
        else if call.method == "queryTimingBloodOxygenMeasureState" {
            //查询定时测血氧状态
            //android 通过CRPBloodOxygenChangeListener回调。onTimingMeasure (int间隔);
            CRPSmartBandSDK.sharedInstance.getAutoO2Interval { data, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 2
                param["timingMeasure"] = data
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodOxygenEventSink?(str)
            }
        }
        else if call.method == "queryTimingBloodOxygen" {
            //查询定时测血氧
            //android 通过CRPBloodOxygenChangeListener回调。onContinueBloodOxygen (CRPBloodOxygenInfo信息);
            //["bloodOxygenTimeType": TODAY]
            //["bloodOxygenTimeType": YESTERDAY]
            let type = receivedDic["bloodOxygenTimeType"] as! String
            CRPSmartBandSDK.sharedInstance.getAutoO2Data { todayData, error in
                //TODO: - flutter监听接收
                if type == "TODAY" {
                    var param: [String: Any] = [:]
                    param["type"] = 2
                    param["continueBO"] = todayData
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.bloodOxygenEventSink?(str)
                }
            } _: { yesterdayData, error in
                //TODO: - flutter监听接收
                if type == "YESTERDAY" {
                    var param: [String: Any] = [:]
                    param["type"] = 2
                    param["continueBO"] = yesterdayData
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.bloodOxygenEventSink?(str)
                }
            }
        }
        else if call.method == "enableContinueBloodOxygen" {
            //设置全天⾎氧
            CRPSmartBandSDK.sharedInstance.setFullDayO2Status(open: true)
        }
        else if call.method == "disableContinueBloodOxygen" {
            //设置全天⾎氧
            CRPSmartBandSDK.sharedInstance.setFullDayO2Status(open: false)
        }
        else if call.method == "queryContinueBloodOxygenState" {
            //查询持续的血氧状态
            //android 通过CRPBloodOxygenChangeListener回调。onContinueState(布尔状态);
            CRPSmartBandSDK.sharedInstance.getFullDayO2Status { data, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 1
                param["continueState"] = Bool(truncating: data as NSNumber)
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodOxygenEventSink?(str)
            }
        }
        else if call.method == "queryLast24HourBloodOxygen" {
            //查询最近24小时血氧 Android
            CRPSmartBandSDK.sharedInstance.getFullDayO2Data { data, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 2
                param["continueBO"] = data
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodOxygenEventSink?(str)
            }
        }
        else if call.method == "queryHistoryBloodOxygen" {
            //获取单次⾎氧测量历史记录
            //android 通过CRPBloodOxygenChangeListener回调。onHistoryBloodOxygen列表(列表);
            CRPSmartBandSDK.sharedInstance.getO2RecordData { datas, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                var list: [[String: Any]] = [[String: Any]]()
                for data in datas {
                    var dic: [String: Any] = [:]
                    dic["bo"] = data.value
                    dic["date"] = String(data.time)
                    list.append(dic)
                }
                param["type"] = 4
                param["historyList"] = list
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.bloodOxygenEventSink?(str)
            }
        }
        //MARK: - 拍照 -
        else if call.method == "enterCameraView" {
            //进入相机
            CRPSmartBandSDK.sharedInstance.switchCameraView()
        }
        else if call.method == "exitCameraView" {
            //退出相机
            CRPSmartBandSDK.sharedInstance.exitCameraView()
        }
        //MARK: - RSSI - iOS端暂时没有
        else if call.method == "readDeviceRssi" {
            //读取手表的实时RSSI   Android
            
        }
        //MARK: - 心电图测量 -
        else if call.method == "startECGMeasure" {
            //开始测量心电图
            CRPSmartBandSDK.sharedInstance.startECGMeasure()
        }
        else if call.method == "stopECGMeasure" {
            //停⽌⼼电测量
            CRPSmartBandSDK.sharedInstance.stopECGMeasure()
        }
        else if call.method == "isNewECGMeasurementVersion" {
            //检测是否是新的⼼电测量⽅式
            let isNew = CRPSmartBandSDK.sharedInstance.isNewECGMeasurementVersion()
            result(isNew)
            
        }
        else if call.method == "queryLastMeasureECGData" {
            //查询上次⼼电数据 android
            //查询手表保存的心电数据，结果由CRPBleECGChangeListener.onECGChange()回调。android
            //TODO: - 通过代理receiveECGDate返回给监听,ECGEvnentSink,flutter监听接收
            CRPSmartBandSDK.sharedInstance.getLastMeasureECGData()
        }
        else if call.method == "sendECGHeartRate" {
            //设置⼼电测量期间⼼率,⽤测量所得到的数据，通过⼼电算法库计算出瞬时⼼率，发送⾄⼿环。
            CRPSmartBandSDK.sharedInstance.sendECGHeartRate(heartRate: receivedDic["heartRate"] as! Int)
        }
        //MARK: - iOS端暂无
        else if call.method == "setECGChangeListener" {
            //所有心电相关数据将通过CRPBleECGChangeListener回收。
        }
        //MARK: - 语言 -
        else if call.method == "sendDeviceVersion" {
            //设置⼿环当前语⾔版本,⼿环语⾔版本分中⽂版和国际版，中⽂版只显示中⽂，国际版才能切换外语
            CRPSmartBandSDK.sharedInstance.setLanguageVersion(true)
        }
        else if call.method == "sendDeviceLanguage" {
            //设置⼿环语⾔
            CRPSmartBandSDK.sharedInstance.setLanguage(receivedDic["language"] as! Int)
        }
        else if call.method == "queryDeviceLanguage" {
            //获取当前⼿环语⾔
            var param: [String: Any] = [:]
            CRPSmartBandSDK.sharedInstance.getLanguage { language, error in
                param["type"] = language
            } _: { languages, error in
                param["languageType"] = languages
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 通知 -
        else if call.method == "setNotification" {
            //设置其它消息推送状态,开启或者关闭其它消息推送。
            var datas: [NotificationType] = []
            
            let notifications: [Int] = receivedDic["ints"] as! [Int]
            for notification in notifications {
                let data: NotificationType = NotificationType.init(rawValue: notification) ?? .phone
                datas.append(data)
            }
            CRPSmartBandSDK.sharedInstance.setNotification(datas)
        }
        else if call.method == "getNotification" {
            //查询其它消息推送状态,查询其它消息推送状态
            CRPSmartBandSDK.sharedInstance.getNotifications { datas, error in
                
                let string = datas.map(String.init).joined(separator: ",")
                result(string)
            }
        }
        else if call.method == "sendMessage" {
            //向手表发送各类消息内容。
            
            let dic: [String: Any] = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["messageInfo"] as! String)
            CRPSmartBandSDK.sharedInstance.setMessage(dic["message"] as! String)
        }
        //MARK: - iOS端暂无
        else if call.method == "endCall" {
            //当手表接收到手机类型的信息推送，手表将振动一个固定的时间。调用这个接口来停止手表手表接通或电话挂断时震动。
            
        }
        //MARK: - 久坐提醒 -
        else if call.method == "sendSedentaryReminder" {
            //开启或者关闭久坐提醒。
            CRPSmartBandSDK.sharedInstance.setRemindersToMove(receivedDic["sedentaryReminder"] as! Bool)
        }
        else if call.method == "querySedentaryReminder" {
            //查询久坐提醒状态
            CRPSmartBandSDK.sharedInstance.getRemindersToMove { state, error in
                result(Bool(truncating: state as NSNumber))
            }
        }
        else if call.method == "sendSedentaryReminderPeriod" {
            //设置久坐提醒有效时间段
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["sedentaryReminderPeriodInfo"] as! String)
            let model = SitRemindModel(period: dic["period"] as! Int, steps: dic["steps"] as! Int, startHour: dic["startHour"] as! Int, endHour: dic["endHour"] as! Int)
            CRPSmartBandSDK.sharedInstance.setSitRemind(model)
        }
        else if call.method == "querySedentaryReminderPeriod" {
            //获取久坐提醒时间段
            CRPSmartBandSDK.sharedInstance.getSitRemindInfo { model, error in
                var param: [String: Any] = [:]
                param["startHour"] = model.startHour
                param["period"] = model.period
                param["endHour"] = model.endHour
                param["steps"] = model.steps
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 找寻设备 -
        else if call.method == "findDevice" {
            //查找⼿环，⼿环收到此指令以后会震动⼏秒。
            CRPSmartBandSDK.sharedInstance.setFindDevice()
        }
        //MARK: - 关机 -
        else if call.method == "shutDown" {
            //⼿环关机
            CRPSmartBandSDK.sharedInstance.shutDown()
        }
        //MARK: - 勿扰时段 -
        else if call.method == "sendDoNotDisturbTime" {
            //设置勿扰时段,⼿环⽀持勿扰时段，勿扰时段内不显示消息推送
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["periodTimeInfo"] as! String)
            let model = periodTimeModel(startHour: dic["startHour"] as! Int, startMin: dic["startMinute"] as! Int, endHour: dic["endHour"] as! Int, endMin: dic["endMinute"] as! Int)
            
            CRPSmartBandSDK.sharedInstance.setDisturbTime(model)
        }
        else if call.method == "queryDoNotDisturbTime" {
            //查询⼿环设置的勿扰时段
            CRPSmartBandSDK.sharedInstance.getDisturbTime { model, error in
                var periodTimeInfo: [String: Any] = [:]
                periodTimeInfo["startHour"] = model.startHour
                periodTimeInfo["startMinute"] = model.startMin
                periodTimeInfo["endHour"] = model.endHour
                periodTimeInfo["endMinute"] = model.endMin
                var param: [String: Any] = [:]
                param["periodTimeInfo"] = periodTimeInfo
                // int DO_NOT_DISTRUB_TYPE = 1;int QUICK_VIEW_TYPE = 2;
                param["periodTimeType"] = 1
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 呼吸灯 -  iOS端暂时没有
        else if call.method == "sendBreathingLight" {
            //设置呼吸灯,一些手表支持呼吸灯，并打开或关闭呼吸灯。

        }
        else if call.method == "queryBreathingLight" {
            //获取呼吸灯的状态
            
        }
        //MARK: - 生理周期 -
        else if call.method == "sendMenstrualCycle" {
            //设置⽣理周期提醒
            //TODO: -
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["menstrualCycleBean"] as! String)
            let startDate: Int = Int(dic["startDate"] as! String) ?? 0
            let date: Date = SystemAuth.timeToDate(timeStamp: startDate)
            
            
            let model = Physiological(reminderModels: [], cycleTime: dic["physiologcalPeriod"] as! Int, menstruationTime: dic["menstrualPeriod"] as! Int, lastTimeMonth: SystemAuth.getDateComponent(type: .month, date: date), lastTimeDay: SystemAuth.getDateComponent(type: .day, date: date), remindTimeHour: dic["reminderHour"] as! Int, remindTimeMinute: dic["reminderMinute"] as! Int)
            CRPSmartBandSDK.sharedInstance.setPhysiological(model)
        }
        else if call.method == "queryMenstrualCycle" {
            //查询⽣理周期提醒
            //TODO: - 这里参数对不上
            CRPSmartBandSDK.sharedInstance.getPhysiological { data, error in
                
                let year = SystemAuth.getDateComponent(type: .year, date: Date())
                
                let dateString: String = "\(year):\(data.lastTimeMonth):\(data.lastTimeDay)"
                let date: Date = SystemAuth.timeToStamp(time: dateString)
                let startDate: Int = SystemAuth.getTimeInterval(date: date)
                var param: [String: Any] = [:]
                param["reminderModels"] = data.reminderModels
                param["physiologcalPeriod"] = data.cycleTime
                param["menstrualPeriod"] = data.menstruationTime
                param["startDate"] = String(startDate)
                param["reminderHour"] = data.remindTimeHour
                param["reminderMinute"] = data.remindTimeMinute
                param["menstrualReminder"] = true
                param["ovulationReminder"] = true
                param["ovulationDayReminder"] = true
                param["ovulationEndReminder"] = true
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 查找手机 -
        else if call.method == "startFindPhone" {
            //开始找寻手机
            //收到回叫后，APP会震动并播放铃声提醒，您就可以按以下命令回复手环。一些手镯支持此命令。
            CRPSmartBandSDK.sharedInstance.sendFindingPhone()
        }
        else if call.method == "stopFindPhone" {
            //结束找寻手机
            CRPSmartBandSDK.sharedInstance.stopFindPhone()
        }
        //MARK: - 播放器 - iOS端暂无此接口
        else if call.method == "setPlayerState" {
            //设置播放器状态

        }
        else if call.method == "sendSongTitle" {
            //设置歌的标题

        }
        else if call.method == "sendLyrics" {
            //设置歌词

        }
        else if call.method == "closePlayerControl" {
            //关闭音乐控制
            
        }
        else if call.method == "sendMaxVolume" {
            //设置最大音量

        }
        else if call.method == "sendCurrentVolume" {
            //设置当前音量

        }
        //MARK: - 喝水提醒 -
        else if call.method == "enableDrinkWaterReminder" {
            //设置喝⽔提醒数据
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["drinkWaterPeriodInfo"] as! String)
            let model = drinkWaterRemind(isRemind: true, startHour: dic["startHour"] as! Int, startMinute: dic["startMinute"] as! Int, remindCount: dic["count"] as! Int, remindTimeInterval: dic["period"] as! Int, waterIntake: dic["currentCups"] as! Int)
            CRPSmartBandSDK.sharedInstance.sendDrinkWaterRemind(model)
        }
        else if call.method == "disableDrinkWaterReminder" {
            //设置喝⽔提醒数据
            let model = drinkWaterRemind(isRemind:false, startHour: 0, startMinute:0, remindCount: 0, remindTimeInterval: 0, waterIntake: 0)
            CRPSmartBandSDK.sharedInstance.sendDrinkWaterRemind(model)
        }
        else if call.method == "queryDrinkWaterReminderPeriod" {
            //获取喝⽔提醒数据
            CRPSmartBandSDK.sharedInstance.getDrinkWaterRemind { data, error in
                var param: [String: Any] = [:]
                param["enable"] = data.isRemind
                param["startHour"] = data.startHour
                param["startMinute"] = data.startMinute
                param["count"] = data.remindCount
                param["period"] = data.remindTimeInterval
                param["currentCups"] = data.waterIntake
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 心率预警 -
        else if call.method == "setMaxHeartRate" {
            //设置⼼率预警值
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["maxHeartRateBean"] as! String)
            let model = hrRemind(isRemind: dic["enable"] as! Bool, max: dic["heartRate"] as! Int)
            CRPSmartBandSDK.sharedInstance.sendHeartRateRemind(model)
        }
        else if call.method == "queryMaxHeartRate" {
            //获取⼼率预警值
            
            CRPSmartBandSDK.sharedInstance.getHeartRateRemind { data, error in
                var param: [String: Any] = [:]
                param["enable"] = data.isRemind
                param["heartRate"] = data.max
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
                
            }
        }
        //MARK: - 锻炼 - iOS端暂无
        else if call.method == "startTraining" {
            //开启锻炼
            CRPSmartBandSDK.sharedInstance.setSportState(state: SportType(rawValue: receivedDic["type"] as! Int) ?? SportType.walking)
        }
        else if call.method == "setTrainingState" {
            //设置锻炼状态
            CRPSmartBandSDK.sharedInstance.setSportState(state: SportType(rawValue: receivedDic["type"] as! Int) ?? SportType.walking)
        }
        //MARK: - 协议版本 - iOS端暂无
        else if call.method == "getProtocolVersion" {
            //获取协议版本

        }
        //MARK: - 温度 -
        else if call.method == "startMeasureTemp" {
            //开始单次体温测量
            //Android：在开始测量之后，通过CRPTempChangeListener.onMeasureTemp()回调实时温度，并且测量状态通过CRPTempChangeListener.onMeasureTempState()回调
            //ios则是：receiveRealTimeTemperature回调⽅法返回测量值，receiveTemperature回调⽅法返回测量状态
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.sendSingleTemperatureStart()
        }
        else if call.method == "stopMeasureTemp" {
            //结束单次体温测量
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.sendSingleTemperatureEnd()
        }
        else if call.method == "enableTimingMeasureTemp" {
            //（iOS）开启定时温度测量，receiveRealTimeTemperature回调⽅法返回测量值，receiveTemperature回调⽅法返回测量状态
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.sendAutoTemperature(true)
        }
        else if call.method == "disableTimingMeasureTemp" {
            //禁用定时温度测量
            CRPSmartBandSDK.sharedInstance.sendAutoTemperature(false)
        }
        else if call.method == "queryTimingMeasureTempState" {
            //获取⾃动体温测量开关
            CRPSmartBandSDK.sharedInstance.getAutoTemperatureState { data, error in
                //1：开启；0：关闭 这个回调没走，不知道为什么
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 3
                param["state"] = Bool(truncating: data as NSNumber)
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.temperatureEventSink?(str)
            }
        }
        else if call.method == "queryTimingMeasureTemp" {
            //获取⾃动体温测量数据,每半⼩时⼀个数据
            //手环可以保存最后两天的计时测温数据，并通过回调结果CRPTempChangeListener.onTimingMeasureTemp (CRPTempInfo)。
            //TODO: - flutter监听接收
            let tempTimeType = receivedDic["tempTimeType"] as! String
            CRPSmartBandSDK.sharedInstance.getAutoTemperatureData { todayDatas, error in
                //TODO: - 通过回调传给监听
                if tempTimeType == "TODAY" {
                    
                    var dic: [String: Any] = [:]
                    dic["tempTimeType"] = tempTimeType
                    dic["tempList"] = todayDatas
                    
                    var param: [String: Any] = [:]
                    param["type"] = 4
                    param["tempInfo"] = dic
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.temperatureEventSink?(str)
                }
            } _: { yesterdayDatas, error in
                //TODO: - 通过回调传给监听
                if tempTimeType == "YESTERDAY" {
                    
                    var dic: [String: Any] = [:]
                    dic["tempTimeType"] = tempTimeType
                    dic["tempList"] = yesterdayDatas
                    
                    var param: [String: Any] = [:]
                    param["type"] = 4
                    param["tempInfo"] = dic
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.temperatureEventSink?(str)
                }
            }
        }
        //MARK: - 亮屏时间
        else if call.method == "sendDisplayTime" {
            //设置亮屏时间
            CRPSmartBandSDK.sharedInstance.sendAutoLockTime(receivedDic["time"] as! Int)
        }
        else if call.method == "queryDisplayTime" {
            //获取亮屏时间
            CRPSmartBandSDK.sharedInstance.getAutoLockTime { data, error in
                result(data)
            }
        }
        //MARK: - 洗手提醒 -
        else if call.method == "enableHandWashingReminder" {
            //开启洗⼿提醒
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["handWashingPeriodInfo"] as! String)
            let model = eventRemind(isRemind: dic["enable"] as! Bool, startHour: dic["startHour"] as! Int, startMinute: dic["startMinute"] as! Int, remindCount: dic["count"] as! Int, remindTimeInterval: dic["period"] as! Int)
            CRPSmartBandSDK.sharedInstance.sendHandWashRemind(model)
        }
        else if call.method == "disableHandWashingReminder" {
            //禁用洗手提醒
            let model = eventRemind(isRemind: false, startHour: 0, startMinute: 0, remindCount: 0, remindTimeInterval: 0)
            CRPSmartBandSDK.sharedInstance.sendHandWashRemind(model)
        }
        else if call.method == "queryHandWashingReminderPeriod" {
            //获取洗⼿提醒数据，结果从闭包返回
            CRPSmartBandSDK.sharedInstance.getHandWashRemind { data, error in
                var param: [String: Any] = [:]
                param["enable"] = data.isRemind
                param["startHour"] = data.startHour
                param["startMinute"] = data.startMinute
                param["count"] = data.remindCount
                param["period"] = data.remindTimeInterval
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        //MARK: - 温度 -
        else if call.method == "sendTempUnit" {
            //切换手环的温度系统。
            CRPSmartBandSDK.sharedInstance.setTemperatureUnit(receivedDic["temp"] as! Int)
        }
        else if call.method == "queryTempUnit" {
            //查询系统温度
            CRPSmartBandSDK.sharedInstance.getTemperatureUnit { data, error in
                //TODO: - flutter监听接收
                var param: [String: Any] = [:]
                param["type"] = 2
                param["tempUnit"] = data
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.weatherEventSink?(str)
            }
            
        }
        //MARK: - 亮度 -  iOS端暂无此接口
        else if call.method == "sendBrightness" {
            //设置亮度

        }
        else if call.method == "queryBrightness" {
            //查询亮度
            
        }
        //MARK: - 蓝牙地址 -
        else if call.method == "queryBtAddress" {
            //获取经典的蓝牙地址
            CRPSmartBandSDK.sharedInstance.getMac { data, error in
                result(data)
            }
        }
        //MARK: - 联系人 -
        else if call.method == "checkSupportQuickContact" {
            //获取联系⼈配置项
            CRPSmartBandSDK.sharedInstance.getContactProfile { data, error in
                var param: [String: Any] = [:]
                param["supported"] = true
                param["width"] = data.contactAvatarWidth
                param["height"] = data.contactAvatarHeight
                param["count"] = data.contactMax
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        else if call.method == "queryContactCount" {
            //获取当前联系⼈个数
            CRPSmartBandSDK.sharedInstance.getContactCount { data, error in
                result(data)
            }
        }
        else if call.method == "sendContactAvatar" {
            //设置联系⼈
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["contactAvatarBean"] as! String)
            let profileModel = contactProfileModel(contactMax: 8, contactAvatarWidth: dic["width"] as! Int, contactAvatarHeight: dic["width"] as! Int)
            let contact: CRPContact = CRPContact(contactID: dic["id"] as! Int, fullName: dic["name"] as! String, image: UIImage(data: Data(dic["avatar"] as! Array)) ?? UIImage(), phoneNumber: dic["number"] as! String)
            
            CRPSmartBandSDK.sharedInstance.setContact(profile: profileModel, contacts: [contact])
        }
        else if call.method == "deleteContact" {
            //删除联系人
            CRPSmartBandSDK.sharedInstance.deleteContact(contactID: receivedDic["index"] as! Int)
        }
        //MARK: - iOS端暂无
        else if call.method == "deleteContactAvatar" {
            //删除联系人
            
        }
        else if call.method == "clearContact" {
            //删除联系人
            CRPSmartBandSDK.sharedInstance.cleanAllContact()
        }
        //MARK: - 省电模式 -
        else if call.method == "sendBatterySaving" {
            //设置省电状态
            let enable = receivedDic["enable"] as! Bool
            CRPSmartBandSDK.sharedInstance.setPowerSaveState(open: enable)
        }
        else if call.method == "queryBatterySaving" {
            //获取省电状态
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.getPowerSaveState { data, error in
                self.batterySavingEventSink?(Bool(truncating: data as NSNumber))
            }
        }
        //MARK: - 吃药提醒 -
        else if call.method == "queryPillReminder" {
            //获取当前吃药提醒信息,通过receiveMedicineInfo⽅法返回数据
            //android 通过CRPPillReminderCallback进行回调。onpillremind (int supportCount, List List);
            CRPSmartBandSDK.sharedInstance.getMedicineInfo()
            self.returnReceiveMedicineInfoClosure { max, model in
                
                var list: [[String: Any]] = [[String: Any]]()
                model?.reminderTimes.forEach({ timeModel in
                    var dic: [String: Any] = [:]
                    dic["remindTimeHour"] = timeModel.remindTimeHour
                    dic["remindTimeMinute"] = timeModel.remindTimeMinute
                    dic["pillCount"] = timeModel.pillCount
                    list.append(dic)
                })
            
                var info: [String: Any] = [:]
                info["id"] = model?.medincineID
                info["dateOffset"] = model?.pastDay
                info["name"] = model?.medincineName
                info["repeat"] = model?.cycleTime
                info["reminderTimeList"] = list
                
                var param: [String: Any] = [:]
                param["supportCount"] = max
                param["list"] = [info]
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                result(str)
            }
        }
        else if call.method == "sendPillReminder" {
            //设置吃药提醒
            //TODO: - 这里有个reminderTimeList不知道
            let dic = SystemAuth.getDictionaryFromJSONString(jsonString: receivedDic["pillReminderInfo"] as! String)
            let model = CRPMedicineReminderModel(medincineID: dic["id"] as! Int, pastDay: dic["dateOffset"] as! Int, medincineName: dic["name"] as! String, cycleTime: dic["repeat"] as! Int, reminderTimes: [])//reminderTimeList
            CRPSmartBandSDK.sharedInstance.setMedicine(medicine: model)
        }
        else if call.method == "deletePillReminder" {
            //删除指定id的吃药提醒
            CRPSmartBandSDK.sharedInstance.deleteMedicine(id: receivedDic["id"] as! Int)
        }
        else if call.method == "clearPillReminder" {
            //删除所有吃药提醒设置
            CRPSmartBandSDK.sharedInstance.deleteAllMedicine()
        }
        //MARK: - 唤醒 -
        else if call.method == "sendWakeState" {
            //设置屏幕轻触唤醒
            CRPSmartBandSDK.sharedInstance.setTapToWake(open: receivedDic["enable"] as! Bool)
        }
        else if call.method == "queryWakeState" {
            //获取屏幕轻触唤醒设置(0: 关闭，1: 开启)
            CRPSmartBandSDK.sharedInstance.readTapToWake { data, error in
                result(Bool(truncating: data as NSNumber))
            }
        }
        //MARK: - 锻炼 - ios端暂无
        else if call.method == "queryHistoryTraining" {
            //查询历史锻炼
            //android 查询手表中存储的培训记录。查询结果将通过CRPTrainingChangeListener。onHistoryTrainingChange列表(列表)。
            //TODO: - flutter监听接收
//            CRPSmartBandSDK.sharedInstance.getSportRecordList()
        }
        //MARK: - iOS端暂无
        else if call.method == "queryTraining" {
            //查询锻炼细节
            
        }
        //MARK: - 睡眠 -
        else if call.method == "querySleep" {
            //同步睡眠
            //获取今天的睡眠数据，结果被CRPSleepChangeListener.onSleepChange()回调。手表的睡眠时间是晚上八点，而且手表会记录从晚上8点到第二天早上10点的睡眠时间(android)
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.getSleepData { data, error in
                
                var soberTime = 0
                for sleepDetail in data.detail {
                    if Int(sleepDetail["type"] ?? "0") == 0 {
                        soberTime += Int(sleepDetail["total"] ?? "0") ?? 0
                    }
                }
                
                var dic: [String: Any] = [:]
                dic["totalTime"] = data.deep + data.light + soberTime
                dic["restfulTime"] = data.deep
                dic["lightTime"] = data.light
                dic["soberTime"] = soberTime
                dic["remTime"] = 0

                var param: [String: Any] = [:]
                param["type"] = 1
                param["sleepInfo"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.sleepEventSink?(str)
            }
            
        }
        else if call.method == "queryRemSleep" {
            //同步rem睡眠
            //增加rem睡眠状态
            //TODO: - flutter监听接收
            CRPSmartBandSDK.sharedInstance.getSleepDataWithREM { data, error in
                
                var soberTime = 0
                for sleepDetail in data.detail {
                    if Int(sleepDetail["type"] ?? "0") == 0 {
                        soberTime += Int(sleepDetail["total"] ?? "0") ?? 0
                    }
                }
                
                var dic: [String: Any] = [:]
                dic["totalTime"] = data.deep + data.light + soberTime
                dic["restfulTime"] = data.deep
                dic["lightTime"] = data.light
                dic["soberTime"] = soberTime
                dic["remTime"] = 0

                var param: [String: Any] = [:]
                param["type"] = 1
                param["sleepInfo"] = dic
                let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                self.sleepEventSink?(str)
            }

        }
        else if call.method == "queryHistorySleep" {
            //同步过去的睡眠
            //会传<String, int>{"pastTimeType": pastTimeType}过来
            //TODO: - flutter监听接收
            let type: Int = receivedDic["historyTimeType"] as! Int
            CRPSmartBandSDK.sharedInstance.getAllData { stepModels, sleepModels, error in
                //⼿环可保存前两天的计步数据和睡眠数据(allDataHandler中包含stepModels和sleepModels,数组的第⼀个为昨天数据，第⼆个为前天数据)
                //TODO: flutter是通过监听接收的
                if type == 3 {
                    //昨天的睡眠
                    let sleepModel0 = sleepModels[0]
                    var soberTime = 0
                    for sleepDetail in sleepModel0.detail {
                        if Int(sleepDetail["type"] ?? "0") == 0 {
                            soberTime += Int(sleepDetail["total"] ?? "0") ?? 0
                        }
                    }
                    
                    var dic: [String: Any] = [:]
                    dic["totalTime"] = sleepModel0.deep + sleepModel0.light + soberTime
                    dic["restfulTime"] = sleepModel0.deep
                    dic["lightTime"] = sleepModel0.light
                    dic["soberTime"] = soberTime
                    dic["remTime"] = 0
                    
                    var historySleep: [String: Any] = [:]
                    historySleep["timeType"] = 3
                    historySleep["sleepInfo"] = dic
                    
                    var param: [String: Any] = [:]
                    param["type"] = 2
                    param["historySleep"] = historySleep
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.sleepEventSink?(str)
                }
                else if type == 4 {
                    //前天的睡眠
                    let sleepModel0 = sleepModels[0]
                    var soberTime = 0
                    for sleepDetail in sleepModel0.detail {
                        if Int(sleepDetail["type"] ?? "0") == 0 {
                            soberTime += Int(sleepDetail["total"] ?? "0") ?? 0
                        }
                    }
                    
                    var dic: [String: Any] = [:]
                    dic["totalTime"] = sleepModel0.deep + sleepModel0.light + soberTime
                    dic["restfulTime"] = sleepModel0.deep
                    dic["lightTime"] = sleepModel0.light
                    dic["soberTime"] = soberTime
                    dic["remTime"] = 0
                    
                    var historySleep: [String: Any] = [:]
                    historySleep["timeType"] = 4
                    historySleep["sleepInfo"] = dic
                    
                    var param: [String: Any] = [:]
                    param["type"] = 2
                    param["historySleep"] = historySleep
                    let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
                    self.sleepEventSink?(str)
                }
            }
        }
        
    }
    
    //MARK: - CRPSmartBand Delegate
    
    /// 返回当前的⼿环的连接状态
    public func didState(_ state: CRPState) {
        self.deviceState = state
        self.connectionStateEventSink?(state.rawValue)
    }
    
    /// 返回当前蓝⽛状态
    public func didBluetoothState(_ state: CRPBluetoothState) {
        
    }
    
    /// 接收实时的运动计步数据
    public func receiveSteps(_ model: StepModel) {
        var dic: [String: Any] = [:]
        dic["calories"] = model.calory
        dic["distance"] = model.distance
        dic["steps"] = model.steps
        dic["time"] = model.time
        
        var param: [String: Any] = [:]
        param["timeType"] = 0
        param["stepsInfo"] = dic
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.stepEventSink?(str)
    }
    
    /// 接收⼼率测量结果(单次测量⼼率)
    public func receiveHeartRate(_ heartRate: Int) {
        var param: [String: Any] = [:]
        param["type"] = 2
        param["onceMeasureComplete"] = heartRate
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.heartRateEventSink?(str)
    }
    
    /// 接收实时⼼率数据(⼀般只⽤于显示不⽤于存储，部分⼿环⽀持返回实时RRI数据)
    public func receiveRealTimeHeartRate(_ heartRate: Int, _ rri: Int) {
        var param: [String: Any] = [:]
        param["type"] = 2
        param["onceMeasureComplete"] = heartRate
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.heartRateEventSink?(str)
    }
    
    /// 查询上次动态⼼率测量结果
    public func receiveHeartRateAll(_ model: HeartModel) {
        var param: [String: Any] = [:]
        param["type"] = 1
        param["measuring"] = model.detail[0]
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.heartRateEventSink?(str)
    }
    
    /// 接收⾎压测量结果
    public func receiveBloodPressure(_ heartRate: Int, _ sbp: Int, _ dbp: Int) {
        
        var dic: [String: Any] = [:]
        dic["sbp"] = sbp
        dic["dbp"] = dbp
        
        var param: [String: Any] = [:]
        param["type"] = 2
        param["pressureChange"] = dic
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.bloodPressureEventSink?(str)
    }
    
    /// 接收⾎氧测量结果
    public func receiveSpO2(_ o2: Int) {
        var param: [String: Any] = [:]
        param["type"] = 3
        param["bloodOxygen"] = o2
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        bloodOxygenEventSink?(str)
    }
    
    public func receiveUpgrede(_ state: CRPUpgradeState, _ progress: Int) {
        
    }
    /// 升级表盘文件/图片时的回调
    public func receiveUpgradeScreen(_ state: CRPUpgradeState, _ progress: Int) {
        
    }
    /// 收到拍照请求
    public func recevieTakePhoto() {
        
    }
    
    /// 收到天⽓获取请求
    /// - Parameter closure:
    public func recevieWeather() {
        
    }

    /// 收到查找⼿机请求
    /// - Parameter closure: state
    public func recevieFindPhone(_ state: Int) {
        
    }

    /// 收到实时的体温数值(
    /// - Parameter ⼿环收到实时体温数值后，保留最后⼀次收到的值，当收到测量结束状态时，最后的值就是测量结果
    public func receiveRealTimeTemperature(_ temp: Double) {
        var param: [String: Any] = [:]
        param["type"] = 2
        param["temp"] = temp
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        temperatureEventSink?(str)
    }

    /// 收到体温测量状态
    /// - Parameter closure: ⽤于判断单次体温测量的状态 收到state为1时为正在测量，收到state为0时为测量结束
    public func receiveTemperature(_ state: Int) {
        var param: [String: Any] = [:]
        param["type"] = 3
        param["state"] = state
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        temperatureEventSink?(str)
    }

    /// 收到⼀键呼叫请求
    public func receiveCalling() {
        
    }

    /// 收到HRV实时数据
    /// - Parameter model: CRPHRVDataModel
    public func receviceHRVRealTime(_ model: CRPHRVDataModel) {
        
    }

    /// 收到当前来电号码
    /// - Parameter number: String
    public func receivePhoneNumber(number: String) {
        
    }

    /// 收到吃药提醒数据
    /// - Parameter closure: _ max: Int,_ model: CRPMedicineReminderModel
    public func receiveMedicineInfo(_ max: Int, _ model: CRPMedicineReminderModel?) {
        if self.receiveMedicineInfoClosure != nil {
            self.receiveMedicineInfoClosure(max, model)
        }
    }

    /// 收到运动模式状态
    /// - Parameter state: SportType
    public func receiveSportState(_ state: SportType) {
        self.trainingStateEventSink?(state.rawValue)
    }



    public func receiveECGDate(_ state: ecgState, _ data: [UInt32], completeTime: Int) {
        var param: [String: Any] = [:]
        param["type"] = state.rawValue
        param["ints"] = data
        param["date"] = completeTime
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        ecgEventSink?(str)
    }
    
    public func receiveSportList(_ list: [CRPSportRecord]) {
        var arr: [[String: Any]] = [[String: Any]]()
        for record in list {
            var dic: [String: Any] = [:]
            dic["type"] = record.type
            dic["startTime"] = record.startTime
            arr.append(dic)
        }
        var param: [String: Any] = [:]
        param["type"] = 1 //1:historyTrainingChange, 2: trainingChange
        param["historyTrainList"] = arr
        let str = SystemAuth.getJsonStringFromBasicTypeData(data: param)
        self.trainEventSink?(str)
    }
    
    
}

